namespace Boxed.AspNetCore.TagHelpers.OpenGraph
{
    using System;
    using System.Collections.Generic;
    using System.Text;
    using Microsoft.AspNetCore.Razor.TagHelpers;

    /// <summary>
    /// This object represents a single author of a book. This object type is not part of the Open Graph standard but
    /// is used by Facebook.
    /// See https://developers.facebook.com/docs/reference/opengraph/object-type/books.author/.
    /// </summary>
    [HtmlTargetElement(
        "open-graph-books-author",
        Attributes = TitleAttributeName + "," + MainImageAttributeName,
        TagStructure = TagStructure.WithoutEndTag)]
    public class OpenGraphBooksAuthor : OpenGraphMetadata
    {
        private const string BookUrlsAttributeName = "book-urls";
        private const string GenderAttributeName = "gender";
        private const string GenreUrlsAttributeName = "gender-urls";
        private const string OfficialSiteUrlAttributeName = "official-site-url";

        /// <summary>
        /// Gets or sets the URL's to the pages about the books written by the author. This URL must contain books.book meta tags <see cref="OpenGraphBooksBook"/>.
        /// </summary>
        [HtmlAttributeName(BookUrlsAttributeName)]
        public IEnumerable<string>? BookUrls { get; set; }

        /// <summary>
        /// Gets or sets the authors gender.
        /// </summary>
        [HtmlAttributeName(GenderAttributeName)]
        public OpenGraphGender? Gender { get; set; }

        /// <summary>
        /// Gets or sets the URL's to the pages about the genres of books the author typically writes. This URL must contain books.genre meta tags <see cref="OpenGraphBooksGenre"/>.
        /// </summary>
        [HtmlAttributeName(GenreUrlsAttributeName)]
        public IEnumerable<string>? GenreUrls { get; set; }

        /// <summary>
        /// Gets the namespace of this open graph type.
        /// </summary>
        public override string Namespace => "books: http://ogp.me/ns/books#";

        /// <summary>
        /// Gets or sets the official site URL of the author.
        /// </summary>
        [HtmlAttributeName(OfficialSiteUrlAttributeName)]
        public Uri? OfficialSiteUrl { get; set; }

        /// <summary>
        /// Gets the type of your object. Depending on the type you specify, other properties may also be required.
        /// </summary>
        public override OpenGraphType Type => OpenGraphType.BooksAuthor;

        /// <summary>
        /// Appends a HTML-encoded string representing this instance to the <paramref name="stringBuilder"/> containing the Open Graph meta tags.
        /// </summary>
        /// <param name="stringBuilder">The string builder.</param>
        public override void ToString(StringBuilder stringBuilder)
        {
            if (stringBuilder is null)
            {
                throw new ArgumentNullException(nameof(stringBuilder));
            }

            base.ToString(stringBuilder);

            stringBuilder.AppendMetaPropertyContentIfNotNull("books:book", this.BookUrls);
            stringBuilder.AppendMetaPropertyContentIfNotNull("books:genre", this.GenreUrls);

            if (this.Gender.HasValue)
            {
                stringBuilder.AppendMetaPropertyContent("books:gender", this.Gender.Value.ToLowercaseString());
            }

            stringBuilder.AppendMetaPropertyContentIfNotNull("books:official_site", this.OfficialSiteUrl);
        }
    }
}
